@@ -3,11 +3,16 @@ import copy
 # from sfsimodels.loader import add_inputs_to_object
 import types
 from sfsimodels.exceptions import ModelError
+from sfsimodels import functions as sf
+import uuid
 
 
 class PhysicalObject(object):
     _counter = 0
     type = "physical_object"
+    _unique_hash = None
+    # inputs = ()
+    skip_list = ()
 
     def __iter__(self):  # real signature unknown
         return self
@@ -69,6 +74,27 @@ class PhysicalObject(object):
                 if update_inputs and item not in self.inputs:
                     self.inputs.append(item)
 
+    def to_dict(self, extra=(), **kwargs):
+        outputs = OrderedDict()
+        export_none = kwargs.get("export_none", True)
+        if hasattr(self, "inputs"):
+            full_inputs = list(self.inputs) + list(extra)
+        else:
+            full_inputs = list(extra)
+        for item in full_inputs:
+            if item not in self.skip_list:
+                value = self.__getattribute__(item)
+                if not export_none and value is None:
+                    continue
+                outputs[item] = sf.collect_serial_value(value)
+        return outputs
+
+    @property
+    def unique_hash(self):
+        if self._unique_hash is None:
+            self._unique_hash = uuid.uuid1()
+        return self._unique_hash
+
 
 class CustomObject(PhysicalObject):
     """
@@ -87,18 +113,6 @@ class CustomObject(PhysicalObject):
             "type"
             ]
 
-    def to_dict(self):
-        outputs = OrderedDict()
-        skip_list = []
-        for item in self.inputs:
-            if item not in skip_list:
-                value = self.__getattribute__(item)
-                if isinstance(value, int):
-                    outputs[item] = str(value)
-                else:
-                    outputs[item] = value
-        return outputs
-
     @property
     def id(self):
         """
