@@ -12,6 +12,12 @@ from sfsimodels import functions as sf
 class Building(PhysicalObject):
     """
     An object to define Buildings
+
+    Parameters
+    ----------
+    n_storeys : int
+        Number of storeys
+
     """
     _id = None
     name = None
@@ -26,7 +32,6 @@ class Building(PhysicalObject):
     _g = 9.81  # m/s2  # gravity
 
     def __init__(self, n_storeys, verbose=0, **kwargs):
-        print(**kwargs)
         super(Building, self).__init__()
         self._n_storeys = n_storeys
         if not hasattr(self, "inputs"):
@@ -34,7 +39,9 @@ class Building(PhysicalObject):
         self._extra_class_variables = [
                 "id",
                 "name",
+                "base_type",
                 "type",
+                'n_storeys',
                 'floor_length',
                 'floor_width',
                 'interstorey_heights',
@@ -42,19 +49,8 @@ class Building(PhysicalObject):
             ]
         self.inputs += self._extra_class_variables
         self.all_parameters = self.inputs + [
-            "n_storeys"
         ]
 
-    def to_dict(self, extra=()):
-        outputs = OrderedDict()
-        skip_list = []
-        full_inputs = self.inputs + list(extra)
-        for item in full_inputs:
-            if item not in skip_list:
-                value = self.__getattribute__(item)
-                outputs[item] = sf.collect_serial_value(value)
-        return outputs
-
     @property
     def ancestor_types(self):
         return super(Building, self).ancestor_types + ["building"]
@@ -114,6 +110,10 @@ class Building(PhysicalObject):
     def n_storeys(self):
         return self._n_storeys
 
+    @n_storeys.setter
+    def n_storeys(self, value):
+        assert self._n_storeys == value
+
     @property
     def interstorey_heights(self):
         return self._interstorey_heights
@@ -153,16 +153,6 @@ class Section(PhysicalObject):  # not used?
         self.inputs = ["depth",
                        "width"]
 
-    def to_dict(self, extra=()):
-        outputs = OrderedDict()
-        skip_list = []
-        full_inputs = self.inputs + list(extra)
-        for item in full_inputs:
-            if item not in skip_list:
-                value = self.__getattribute__(item)
-                outputs[item] = sf.collect_serial_value(value)
-        return outputs
-
     @property
     def depth(self):
         return self._depth
@@ -183,8 +173,11 @@ class Section(PhysicalObject):  # not used?
 class Element(PhysicalObject):
     section_lengths = None
 
-    def __init__(self):
-        self.sections = [Section()]
+    def __init__(self, section_class=None):
+        if section_class is None:
+            self.sections = [Section()]
+        else:
+            self.sections = [section_class()]
 
     @property
     def s(self):
@@ -213,7 +206,7 @@ class Element(PhysicalObject):
     def get_section_prop(self, prop, section_i=0):
         return getattr(self.sections[section_i], prop)
 
-    def to_dict(self, extra=()):
+    def to_dict(self, extra=(), **kwargs):
         output = []
         for i in range(len(self.sections)):
             output.append(self.sections[i].to_dict(extra=extra))
@@ -222,11 +215,19 @@ class Element(PhysicalObject):
 
 class Frame(object):
     _bay_lengths = None
+    _custom_beam_section = None
+    _custom_column_section = None
+    _loaded_beam_section_ids = None  # should not be accessed by end user
+    _loaded_beam_sections = None  # should not be accessed by end user
+    _loaded_column_section_ids = None  # should not be accessed by end user
+    _loaded_column_sections = None  # should not be accessed by end user
 
     def __init__(self, n_storeys, n_bays):
         if not hasattr(self, "inputs"):
             self.inputs = []
         self._extra_class_variables = [
+            "n_bays",
+            "n_storeys",
             "beams",
             "columns",
             "bay_lengths"
@@ -236,7 +237,7 @@ class Frame(object):
         self._n_bays = n_bays
         self._allocate_beams_and_columns()
 
-    def to_dict(self, extra=()):
+    def to_dict(self, extra=(), **kwargs):
         outputs = OrderedDict()
         skip_list = []
         # skip_list = ["beams", "columns"]  # TODO: uncomment this
@@ -277,8 +278,8 @@ class Frame(object):
         return ["frame"]
 
     def _allocate_beams_and_columns(self):
-        self._beams = [[Element() for i in range(self.n_bays)] for ss in range(self.n_storeys)]
-        self._columns = [[Element() for i in range(self.n_cols)] for ss in range(self.n_storeys)]
+        self._beams = [[Element(self._custom_beam_section) for i in range(self.n_bays)] for ss in range(self.n_storeys)]
+        self._columns = [[Element(self._custom_column_section) for i in range(self.n_cols)] for ss in range(self.n_storeys)]
 
     @property
     def beams(self):
@@ -292,17 +293,96 @@ class Frame(object):
     def n_bays(self):
         return self._n_bays
 
+    @n_bays.setter
+    def n_bays(self, value):
+        assert self._n_bays == value
+
     @property
     def n_storeys(self):
         return self._n_storeys
 
+    @n_storeys.setter
+    def n_storeys(self, value):
+        assert self._n_storeys == value
+
     @property
     def n_cols(self):
         return self._n_bays + 1
 
-    @n_bays.setter
-    def n_bays(self, value):
-        raise ModelError("Can not set n_bays, only on initialisation")
+    @property
+    def beam_section_ids(self):
+        return None
+
+    @beam_section_ids.setter
+    def beam_section_ids(self, beam_ids):
+        self._loaded_beam_section_ids = beam_ids
+        if self._loaded_beam_sections is not None:
+            self._assign_loaded_beams()
+
+    @property
+    def beam_sections(self):
+        return None
+
+    @beam_sections.setter
+    def beam_sections(self, beam_sections):
+        self._loaded_beam_sections = beam_sections
+        if self._loaded_beam_section_ids is not None:
+            self._assign_loaded_beams()
+
+    def _assign_loaded_beams(self):
+        for ss in range(self.n_storeys):
+            for bb in range(self.n_bays):
+                sect_is = self._loaded_beam_section_ids[ss][bb]
+                if hasattr(sect_is, "__len__"):
+                    n_sections = len(sect_is)
+                    self.beams[ss][bb].split_into_multiple([1] * n_sections)  # TODO: should be lengths
+                    for sect_i in range(len(sect_is)):
+                        beam_sect_id = str(self._loaded_beam_section_ids[ss][bb][sect_i])
+                        sect_dictionary = self._loaded_beam_sections[beam_sect_id]
+                        sf.add_to_obj(self.beams[ss][bb].sections[sect_i], sect_dictionary)
+                else:  # deprecated loading
+                    deprecation("Frame data structure is out-of-date, please load and save the file to update.")
+                    beam_sect_id = str(self._loaded_beam_section_ids[ss][bb])
+                    sect_dictionary = self._loaded_beam_sections[beam_sect_id]
+                    sf.add_to_obj(self.beams[ss][bb].sections[0], sect_dictionary)
+
+    @property
+    def column_section_ids(self):
+        return None
+
+    @column_section_ids.setter
+    def column_section_ids(self, column_ids):
+        self._loaded_column_section_ids = column_ids
+        if self._loaded_column_sections is not None:
+            self._assign_loaded_columns()
+
+    @property
+    def column_sections(self):
+        return None
+
+    @column_sections.setter
+    def column_sections(self, column_sections):
+        self._loaded_column_sections = column_sections
+        if self._loaded_column_section_ids is not None:
+            self._assign_loaded_columns()
+
+    def _assign_loaded_columns(self):
+        for ss in range(self.n_storeys):
+            for bb in range(self.n_bays):
+                sect_is = self._loaded_column_section_ids[ss][bb]
+                if hasattr(sect_is, "__len__"):
+                    n_sections = len(sect_is)
+                    self.columns[ss][bb].split_into_multiple([1] * n_sections)  # TODO: should be lengths
+                    for sect_i in range(len(sect_is)):
+                        column_sect_id = str(self._loaded_column_section_ids[ss][bb][sect_i])
+                        sect_dictionary = self._loaded_column_sections[column_sect_id]
+                        sf.add_to_obj(self.columns[ss][bb].sections[sect_i], sect_dictionary)
+                else:  # deprecated loading
+                    deprecation("Frame data structure is out-of-date, "
+                                "run sfsimodels.migrate_ecp(<file-path>, <out-file-path>).")
+                    column_sect_id = str(self._loaded_column_section_ids[ss][bb])
+                    sect_dictionary = self._loaded_column_sections[column_sect_id]
+                    sf.add_to_obj(self.columns[ss][bb].sections[0], sect_dictionary)
 
     # @column_sections.setter
     # def column_sections(self, columns: dict):
@@ -398,10 +478,10 @@ class Frame(object):
         self._bay_lengths = np.array(bay_lengths)
 
 
-class BuildingFrame(Frame, Building):
+class FrameBuilding(Frame, Building):
     _n_seismic_frames = None
     _n_gravity_frames = None
-    type = "building_frame"
+    type = "frame_building"
 
     def __init__(self, n_storeys, n_bays):
         Frame.__init__(self, n_storeys, n_bays)
@@ -415,7 +495,7 @@ class BuildingFrame(Frame, Building):
 
     @property
     def ancestor_types(self):
-        return super(BuildingFrame, self).ancestor_types + ["building_frame"]
+        return super(FrameBuilding, self).ancestor_types + ["frame_building"]
 
     @property
     def n_seismic_frames(self):
@@ -434,15 +514,15 @@ class BuildingFrame(Frame, Building):
         self._n_gravity_frames = value
 
 
-class FrameBuilding(BuildingFrame):
+class BuildingFrame(FrameBuilding):
     def __init__(self, n_storeys, n_bays):
-        deprecation("FrameBuilding is deprecated, use BuildingFrame.")
-        super(FrameBuilding, self).__init__(n_storeys, n_bays)
+        deprecation("BuildingFrame is deprecated, use FrameBuilding.")
+        super(BuildingFrame, self).__init__(n_storeys, n_bays)
 
 
-class BuildingFrame2D(Frame, Building):
+class FrameBuilding2D(Frame, Building):
     _extra_class_inputs = []
-    type = "building_frame2D"
+    type = "frame_building2D"
 
     def __init__(self, n_storeys, n_bays):
         Frame.__init__(self, n_storeys, n_bays)
@@ -452,19 +532,9 @@ class BuildingFrame2D(Frame, Building):
 
     @property
     def ancestor_types(self):
-        return ["physical_object", "frame", "building"] + ["building_frame2D"]  # TODO: improve this logic
+        return ["physical_object", "frame", "building"] + ["frame_building2D"]  # TODO: improve this logic
 
-    # def to_dict(self, extra=()):
-    #     outputs = OrderedDict()
-    #     skip_list = []
-    #     full_inputs = self.inputs + list(extra)
-    #     for item in full_inputs:
-    #         if item not in skip_list:
-    #             value = self.__getattribute__(item)
-    #             outputs[item] = sf.collect_serial_value(value)
-    #     return outputs
-
-    def to_dict(self, extra=()):
+    def to_dict(self, extra=(), compression=True, **kwargs):
         outputs = OrderedDict()
         skip_list = ["beams", "columns"]
         full_inputs = self.inputs + list(extra)
@@ -480,19 +550,27 @@ class BuildingFrame2D(Frame, Building):
         for ss in range(self.n_storeys):
             column_section_ids.append([])
             for cc in range(self.n_cols):
-                # build a hash string of the section inputs to check uniqueness
-                parts = []
+
                 column_section_ids[ss].append([])
                 for sect_i in range(len(self.columns[ss][cc].sections)):
-                    for item in self.columns[ss][cc].sections[sect_i].inputs:
-                        parts.append(item)
-                        parts.append(str(self.columns[ss][cc].get_section_prop(item, section_i=sect_i)))
-                    p_str = "-".join(parts)
+                    if compression:  # build a hash string of the section inputs to check uniqueness
+                        parts = []
+                        for item in self.columns[ss][cc].sections[sect_i].inputs:
+                            if item == "id" or item == "name":
+                                continue
+                            parts.append(item)
+                            parts.append(str(self.columns[ss][cc].get_section_prop(item, section_i=sect_i)))
+                        p_str = "-".join(parts)
+                    else:
+                        p_str = str(sect_i)
                     if p_str not in column_sections:
                         column_sections[p_str] = self.columns[ss][cc].sections[sect_i].to_dict(extra)
                         column_section_count += 1
-                        column_sections[p_str]["id"] = column_section_count
-                    column_section_ids[ss][cc].append(column_section_count)
+                        col_sect_id = column_section_count
+                        column_sections[p_str]["id"] = col_sect_id
+                    else:
+                        col_sect_id = column_sections[p_str]["id"]
+                    column_section_ids[ss][cc].append(col_sect_id)
 
         beam_sections = OrderedDict()
         beam_section_ids = []
@@ -500,19 +578,26 @@ class BuildingFrame2D(Frame, Building):
         for ss in range(self.n_storeys):
             beam_section_ids.append([])
             for bb in range(self.n_bays):
-                # build a hash string of the section inputs to check uniqueness
-                parts = []
                 beam_section_ids[ss].append([])
                 for sect_i in range(len(self.beams[ss][bb].sections)):
-                    for item in self.beams[ss][bb].sections[sect_i].inputs:
-                        parts.append(item)
-                        parts.append(str(self.beams[ss][bb].get_section_prop(item, section_i=sect_i)))
-                    p_str = "-".join(parts)
+                    if compression:  # build a hash string of the section inputs to check uniqueness
+                        parts = []
+                        for item in self.beams[ss][bb].sections[sect_i].inputs:
+                            if item == "id" or item == "name":
+                                continue
+                            parts.append(item)
+                            parts.append(str(self.beams[ss][bb].get_section_prop(item, section_i=sect_i)))
+                        p_str = "-".join(parts)
+                    else:
+                        p_str = str(sect_i)
                     if p_str not in beam_sections:
                         beam_sections[p_str] = self.beams[ss][bb].sections[sect_i].to_dict(extra)
                         beam_section_count += 1
-                        beam_sections[p_str]["id"] = beam_section_count
-                    beam_section_ids[ss][bb].append(beam_section_count)
+                        beam_sect_id = beam_section_count
+                        beam_sections[p_str]["id"] = beam_sect_id
+                    else:
+                        beam_sect_id = beam_sections[p_str]["id"]
+                    beam_section_ids[ss][bb].append(beam_sect_id)
 
         outputs["column_section_ids"] = column_section_ids
         outputs["beam_section_ids"] = beam_section_ids
@@ -525,20 +610,20 @@ class BuildingFrame2D(Frame, Building):
         return outputs
 
 
-class FrameBuilding2D(BuildingFrame2D):
+class BuildingFrame2D(FrameBuilding2D):
     def __init__(self, n_storeys, n_bays):
-        deprecation("FrameBuilding2D class is deprecated, use BuildingFrame2D.")
-        super(FrameBuilding2D, self).__init__(n_storeys, n_bays)
+        deprecation("BuildingFrame2D class is deprecated, use FrameBuilding2D.")
+        super(BuildingFrame2D, self).__init__(n_storeys, n_bays)
 
 
-class BuildingWall(Building):  # new name
+class WallBuilding(Building):  # new name
     n_walls = 1
     wall_depth = 0.0  # m
     wall_width = 0.0  # m
-    type = "building_wall"
+    type = "wall_building"
 
     def __init__(self, n_storeys):
-        super(BuildingWall, self).__init__(n_storeys)  # run parent class initialiser function
+        super(WallBuilding, self).__init__(n_storeys)  # run parent class initialiser function
         self._extra_class_inputs = [
             "n_walls",
             "wall_depth",
@@ -548,16 +633,16 @@ class BuildingWall(Building):  # new name
 
     @property
     def ancestor_types(self):
-        return super(BuildingWall, self).ancestor_types + ["building_wall"]
+        return super(WallBuilding, self).ancestor_types + ["wall_building"]
 
 
-class WallBuilding(BuildingWall):
+class BuildingWall(WallBuilding):
     def __init__(self, n_storeys):
-        deprecation("WallBuilding class is deprecated, use BuildingWall.")
-        super(WallBuilding, self).__init__(n_storeys)
+        deprecation("BuildingWall class is deprecated, use WallBuilding.")
+        super(BuildingWall, self).__init__(n_storeys)
 
 
-class BuildingSDOF(PhysicalObject):
+class SDOFBuilding(PhysicalObject):
     """
     An object to describe structures.
     """
@@ -583,21 +668,9 @@ class BuildingSDOF(PhysicalObject):
         ]
         self._g = g
 
-    def to_dict(self):
-        outputs = OrderedDict()
-        skip_list = []
-        for item in self.inputs:
-            if item not in skip_list:
-                value = self.__getattribute__(item)
-                if isinstance(value, int):
-                    outputs[item] = str(value)
-                else:
-                    outputs[item] = value
-        return outputs
-
     @property
     def ancestor_types(self):
-        return super(BuildingSDOF, self).ancestor_types + ["sdof"]
+        return super(SDOFBuilding, self).ancestor_types + ["sdof"]
 
     @property
     def id(self):
@@ -656,6 +729,12 @@ class BuildingSDOF(PhysicalObject):
         return self.mass_eff / self.mass_ratio * self._g
 
 
+class BuildingSDOF(SDOFBuilding):
+    def __init__(self, g=9.8):
+        deprecation("BuildingSDOF class is deprecated, use SDOFBuilding.")
+        super(BuildingSDOF, self).__init__(g=g)
+
+
 class Structure(BuildingSDOF):
     def __init__(self, g=9.8):
         deprecation("Structure class is deprecated, use BuildingSDOF.")
@@ -663,7 +742,7 @@ class Structure(BuildingSDOF):
 
 
 class SoilStructureSystem(PhysicalObject):
-    bd = BuildingSDOF()
+    bd = SDOFBuilding()
     fd = Foundation()
     sp = Soil()
     hz = SeismicHazard()
