@@ -1,5 +1,5 @@
-import numbers
 from collections import OrderedDict
+from sfsimodels.exceptions import deprecation
 
 import numpy as np
 
@@ -7,6 +7,7 @@ from sfsimodels.exceptions import ModelError, AnalysisError
 from sfsimodels.functions import clean_float
 from sfsimodels.models.abstract_models import PhysicalObject
 from sfsimodels import checking_tools as ct
+from sfsimodels import functions as sf
 
 
 class Soil(PhysicalObject):
@@ -39,8 +40,9 @@ class Soil(PhysicalObject):
     _bulk_mod = None  # Bulk modulus [Pa]
     _poissons_ratio = None
     _plasticity_index = None
+    _gravity = 9.8
 
-    def __init__(self, pw=9800):
+    def __init__(self, pw=9800, **kwargs):
         self._pw = pw  # specific weight of water
         self.stack = []
         self._extra_class_inputs = [
@@ -68,20 +70,14 @@ class Soil(PhysicalObject):
         ]
         if not hasattr(self, "inputs"):
             self.inputs = []
-        self.inputs += self._extra_class_inputs
-
-    def to_dict(self):
-        """
-        Passes all of the inputs and the values to an ordered dictionary
-        :return:
-        """
-        outputs = OrderedDict()
-        for item in self.inputs:
-            outputs[item] = self.__getattribute__(item)
-        return outputs
+        self.inputs += list(self._extra_class_inputs)
+        for param in kwargs:
+            if param in self.inputs:
+                setattr(self, param, kwargs[param])
 
     @property
     def ancestor_types(self):
+        """View list of types from inherited objects"""
         parent_ancestor_types = super(Soil, self).ancestor_types
         return parent_ancestor_types + ["soil"]
 
@@ -125,7 +121,6 @@ class Soil(PhysicalObject):
     def reset_all(self):
         """
         Resets all parameters to None
-        :return:
         """
         for item in self.inputs:
             setattr(self, "_%s" % item, None)
@@ -144,17 +139,14 @@ class Soil(PhysicalObject):
 
     @property
     def id(self):
-        """
-        Object id
-        :return:
-        """
+        """Object id"""
         return self._id
 
     @property
     def unit_weight(self):
         """
         The unit moist weight of the soil (accounts for saturation level)
-        :return:
+        :return: float
         """
         if self.saturation is not None:
             return self.unit_moist_weight
@@ -245,6 +237,43 @@ class Soil(PhysicalObject):
         except TypeError:
             return None
 
+    @property
+    def unit_dry_mass(self):
+        """The mass of the soil in dry state"""
+        try:
+            return self._unit_dry_weight / self._gravity
+        except TypeError:
+            return None
+
+    @property
+    def unit_sat_mass(self):
+        """The mass of the soil when fully saturated"""
+        try:
+            return self._unit_sat_weight / self._gravity
+        except TypeError:
+            return None
+
+    def get_shear_vel(self, saturated):
+        """
+        Calculate the shear wave velocity
+
+        :param saturated: bool, if true then use saturated mass
+        :return:
+        """
+        try:
+            if saturated:
+                return np.sqrt(self.g_mod / self.unit_sat_mass)
+            else:
+                return np.sqrt(self.g_mod / self.unit_dry_mass)
+        except TypeError:
+            return None
+
+    def get_unit_mass(self, saturated):
+        if saturated:
+            return self.unit_sat_mass
+        else:
+            return self.unit_dry_mass
+
     @property
     def permeability(self):
         """The permeability of the soil"""
@@ -692,40 +721,50 @@ class Soil(PhysicalObject):
             return None
 
 
-class SoilCritical(Soil):
+class CriticalSoil(Soil):
     # critical state parameters
     e_cr0 = 0.0
     p_cr0 = 0.0
     lamb_crl = 0.0
-    type = "soil_critical"
+    type = "critical_soil"
 
-    def __init__(self, pw=9800):
-        super(SoilCritical, self).__init__(pw=pw)  # run parent class initialiser function
+    def __init__(self, pw=9800, **kwargs):
+        super(CriticalSoil, self).__init__(pw=pw)  # run parent class initialiser function
         self._extra_class_inputs = ["e_cr0", "p_cr0", "lamb_crl"]
         self.inputs = self.inputs + self._extra_class_inputs
+        for param in kwargs:
+            if param in self.inputs:
+                setattr(self, param, kwargs[param])
 
     @property
     def ancestor_types(self):
-        return super(SoilCritical, self).ancestor_types + [self.type]
+        return super(CriticalSoil, self).ancestor_types + [self.type]
 
     def e_critical(self, p):
         p = float(p)
         return self.e_cr0 - self.lamb_crl * np.log(p / self.p_cr0)
 
 
-class SoilStressDependent(Soil):
+class StressDependentSoil(Soil):
     _g0_mod = None
     _p_atm = 101000.0  # Pa
-    type = "soil_stress_dependent"
+    type = "stress_dependent_soil"
 
-    def __init__(self, pw=9800):
-        super(SoilStressDependent, self).__init__(pw=pw)
+    def __init__(self, pw=9800, **kwargs):
+        super(StressDependentSoil, self).__init__(pw=pw)
         self._extra_class_inputs = ["g0_mod", "p_atm"]
         self.inputs = self.inputs + self._extra_class_inputs
+        for param in kwargs:
+            if param in self.inputs:
+                setattr(self, param, kwargs[param])
 
     @property
     def ancestor_types(self):
-        return super(SoilStressDependent, self).ancestor_types + [self.type]
+        return super(StressDependentSoil, self).ancestor_types + [self.type]
+
+    # @g_mod.setter
+    # def g_mod(self, value):
+    #     raise ValueError
 
     @property
     def g0_mod(self):
@@ -745,12 +784,34 @@ class SoilStressDependent(Soil):
         value = clean_float(value)
         self._p_atm = value
 
-    def g_mod_at_v_eff_stress(self, sigma_v_eff):
+    def get_g_mod_at_v_eff_stress(self, v_eff_stress):
         k0 = 1 - np.sin(self.phi_r)
-        return self.g0_mod * self.p_atm * (sigma_v_eff * (1 + 2 * k0) / 3 / self.p_atm) ** 0.5
+        return self.g0_mod * self.p_atm * (v_eff_stress * (1 + 2 * k0) / 3 / self.p_atm) ** 0.5
 
-    def g_mod_at_m_eff_stress(self, sigma_m_eff):
-        return self.g0_mod * self.p_atm * (sigma_m_eff / self.p_atm) ** 0.5
+    def get_g_mod_at_m_eff_stress(self, m_eff_stress):
+        return self.g0_mod * self.p_atm * (m_eff_stress / self.p_atm) ** 0.5
+
+    def get_shear_vel_at_v_eff_stress(self, v_eff_stress, saturated):
+        try:
+            g_mod = self.get_g_mod_at_v_eff_stress(v_eff_stress)
+            if saturated:
+                return np.sqrt(g_mod / self.unit_sat_mass)
+            else:
+                return np.sqrt(g_mod / self.unit_dry_mass)
+        except TypeError:
+            return None
+
+    def g_mod_at_v_eff_stress(self, v_eff_stress):
+        deprecation("Use get_g_mod_at_v_eff_stress")
+        return self.get_g_mod_at_v_eff_stress(v_eff_stress)
+
+    def g_mod_at_m_eff_stress(self, m_eff_stress):
+        deprecation("Use get_g_mod_at_m_eff_stress")
+        return self.get_g_mod_at_m_eff_stress(m_eff_stress)
+
+    def calc_shear_vel_at_v_eff_stress(self, saturated, v_eff_stress):
+        deprecation("Use get_shear_vel_at_v_eff_stress - note inputs switched")
+        return self.get_shear_vel_at_v_eff_stress(v_eff_stress, saturated)
 
 
 class SoilLayer(Soil):  # not used
@@ -774,6 +835,7 @@ class SoilProfile(PhysicalObject):
     unit_water_weight = 9800.  # [N/m3]
     _height = None
     hydrostatic = False
+    base_type = "soil_profile"
     type = "soil_profile"
 
     inputs = [
@@ -788,11 +850,13 @@ class SoilProfile(PhysicalObject):
     def __init__(self):
         super(PhysicalObject, self).__init__()  # run parent class initialiser function
         self._layers = OrderedDict([(0.0, Soil())])  # [depth to top of layer, Soil object]
+        self.skip_list = ["layers"]
+        self.split = OrderedDict()
 
     def __str__(self):
         return "SoilProfile id: {0}, name: {1}".format(self.id, self.name)
 
-    def to_dict(self):
+    def to_dict(self, **kwargs):
         outputs = OrderedDict()
         skip_list = ["layers"]
         for item in self.inputs:
@@ -807,11 +871,34 @@ class SoilProfile(PhysicalObject):
         #     outputs["layers"].append({"depth": float(depth), "soil": self.layers[depth].to_dict()})
         return outputs
 
+    def add_to_dict(self, models_dict, **kwargs):
+        if self.base_type not in models_dict:
+            models_dict[self.base_type] = OrderedDict()
+        if "soil" not in models_dict:
+            models_dict["soil"] = OrderedDict()
+        profile_dict = self.to_dict()
+        profile_dict["layers"] = []
+        for layer in self.layers:
+            models_dict["soil"][self.layers[layer].unique_hash] = self.layers[layer].to_dict()
+            profile_dict["layers"].append({
+                "soil_id": str(self.layers[layer].id),
+                "depth": float(layer)
+            })
+        models_dict["soil_profile"][self.unique_hash] = profile_dict
+
     @property
     def ancestor_types(self):
         return super(SoilProfile, self).ancestor_types + ["soil_profile"]
 
-    def add_layer(self, depth: float, soil):
+    def add_layer(self, depth, soil):
+        """
+        Adds a soil to the SoilProfile at a set depth.
+
+        Note, the soils are automatically reordered based on depth from surface.
+
+        :param depth: depth from surface to top of soil layer
+        :param soil: Soil object
+        """
 
         self._layers[depth] = soil
         self._sort_layers()
@@ -830,26 +917,37 @@ class SoilProfile(PhysicalObject):
                     soil.saturation = sat_height / self.layer_height(li)
 
     def _sort_layers(self):
-        """
-        Sort the layers by depth.
-        :return:
-        """
+        """Sort the layers by depth."""
         self._layers = OrderedDict(sorted(self._layers.items(), key=lambda t: t[0]))
 
     @property
     def id(self):
+        """Get the id number of the soil profile"""
         return self._id
 
     @id.setter
-    def id(self, value: int):
+    def id(self, value):
+        """
+        Set the id of the soil profile
+
+        :param value: int
+        :return:
+        """
         self._id = int(value)
 
     @property
     def gwl(self):
+        """Get the ground water level"""
         return self._gwl
 
     @gwl.setter
-    def gwl(self, value: float):
+    def gwl(self, value):
+        """
+        Set the depth from the surface to the ground water level (gwl)
+
+        :param value:
+        :return:
+        """
         self._gwl = float(value)
 
     @property
@@ -857,14 +955,21 @@ class SoilProfile(PhysicalObject):
         return self._height
 
     @height.setter
-    def height(self, value: float):
+    def height(self, value):
+        """
+        Sets the depth from the surface to the base of the soil profile
+
+        :param value: float, height
+        :return:
+        """
         self._height = float(value)
 
     def layer_height(self, layer_int):
         """
-        The number of the layer.
+        Get the layer height by layer id number.
+
         :param layer_int:
-        :return:
+        :return: float, height of the soil layer
         """
         if layer_int == self.n_layers:
             if self.height is None:
@@ -877,6 +982,13 @@ class SoilProfile(PhysicalObject):
     def layers(self):
         return self._layers
 
+    @layers.setter
+    def layers(self, layers):
+        for layer in layers:
+            layer_depth = layer["depth"]
+            sl = layer["soil"]  # is actually a soil object
+            self.add_layer(layer_depth, sl)
+
     def remove_layer_at_depth(self, depth):
         try:
             del self._layers[depth]
@@ -954,8 +1066,10 @@ class SoilProfile(PhysicalObject):
     def equivalent_crust_cohesion(self):
         """
         Calculate the equivalent crust cohesion strength according to Karamitros et al. 2013 sett, pg 8 eq. 14
+
         :return: equivalent cohesion [Pa]
         """
+        deprecation("Will be moved to a function")
         if len(self.layers) > 1:
             crust = self.layer(0)
             crust_phi_r = np.radians(crust.phi)
@@ -965,6 +1079,7 @@ class SoilProfile(PhysicalObject):
 
     @property
     def crust_effective_unit_weight(self):
+        deprecation("Will be moved to a function")
         if len(self.layers) > 1:
             crust = self.layer(0)
             crust_height = self.layer_depth(1)
@@ -973,7 +1088,11 @@ class SoilProfile(PhysicalObject):
             unit_weight_eff = (total_stress_base - pore_pressure_base) / crust_height
             return unit_weight_eff
 
-    def vertical_total_stress(self, z):
+    def vertical_total_stress(self, y_c):
+        deprecation("Use get_v_total_stress_at_depth")
+        return self.get_v_total_stress_at_depth(y_c)
+
+    def get_v_total_stress_at_depth(self, z):
         """
         Determine the vertical total stress at depth z, where z can be a number or an array of numbers.
         """
@@ -986,9 +1105,11 @@ class SoilProfile(PhysicalObject):
                 sigma_v_effs.append(self.one_vertical_total_stress(value))
             return np.array(sigma_v_effs)
 
-    def one_vertical_total_stress(self, z_c: float):
+    def one_vertical_total_stress(self, z_c):
         """
         Determine the vertical total stress at a single depth z_c.
+
+        :param z_c: depth from surface
         """
         total_stress = 0.0
         depths = self.depths
@@ -1022,16 +1143,166 @@ class SoilProfile(PhysicalObject):
     def hydrostatic_pressure(self, y_c):
         """
         Determine the vertical effective stress at a single depth y_c.
-        """
-        return max(y_c - self.gwl, 0.0) * self.unit_water_weight
 
-    def vertical_effective_stress(self, y_c):
+        :param y_c: float, depth from surface
+        """
+        deprecation("Use get_hydrostatic_pressure_at_depth")
+        return self.get_hydrostatic_pressure_at_depth(y_c)
+
+    def get_hydrostatic_pressure_at_depth(self, y_c):
+        return np.where(y_c < self.gwl, 0.0, (y_c - self.gwl) * self.unit_water_weight)
+
+    def vert_eff_stress(self, y_c):
         """
         Determine the vertical effective stress at a single depth z_c.
+
+        :param y_c: float, depth from surface
         """
-        sigma_v_c = self.vertical_total_stress(y_c)
-        pp = self.hydrostatic_pressure(y_c)
+        deprecation("Use get_v_eff_stress_at_depth")
+        return self.get_v_eff_stress_at_depth(y_c)
+
+    def get_v_eff_stress_at_depth(self, y_c):
+        """
+        Determine the vertical effective stress at a single depth z_c.
+
+        :param y_c: float, depth from surface
+        """
+        sigma_v_c = self.get_v_total_stress_at_depth(y_c)
+        pp = self.get_hydrostatic_pressure_at_depth(y_c)
         sigma_veff_c = sigma_v_c - pp
         return sigma_veff_c
 
+    def vertical_effective_stress(self, y_c):  # deprecated function
+        """Deprecated. Use get_vert_eff_stress"""
+        deprecation("Use get_v_eff_stress_at_depth")
+        return self.vert_eff_stress(y_c)
+
+    def shear_vel_at_depth(self, y_c):
+        """
+        Get the shear wave velocity at a depth.
+
+        :param y_c: float, depth from surface
+        :return:
+        """
+        sl = self.get_soil_at_depth(y_c)
+        if y_c <= self.gwl:
+            saturation = False
+        else:
+            saturation = True
+        if hasattr(sl, "get_shear_vel_at_v_eff_stress"):
+            v_eff = self.get_v_eff_stress_at_depth(y_c)
+            vs = sl.get_shear_vel_at_v_eff_stress(v_eff, saturation)
+        else:
+            vs = sl.get_shear_vel(saturation)
+        return vs
+
+    def split_props(self, incs=None, target=1.0, props=None):
+        if incs is None:
+            incs = np.ones(self.n_layers) * target
+        if props is None:
+            props = ['unit_mass', 'shear_vel']
+        else:
+            if 'thickness' in props:
+                props.remove('thickness')
+        dd = OrderedDict([('thickness', [])])
+        for item in props:
+            dd[item] = []
+
+        cum_thickness = 0
+        for i in range(self.n_layers):
+            sl = self.layer(i + 1)
+            thickness = self.layer_height(i + 1)
+            if thickness is None:
+                raise ValueError("thickness of layer {0} is None, check if soil_profile.height is set".format(i + 1))
+            n_slices = max(int(thickness / incs[i]), 1)
+            slice_thickness = float(thickness) / n_slices
+            for j in range(n_slices):
+                dd["thickness"].append(slice_thickness)
+                v_eff = None
+                cum_thickness += slice_thickness
+                if cum_thickness >= self.gwl:
+                    saturated = True
+                else:
+                    saturated = False
+                # some properties require vertical effective stress or saturation
+                for item in props:
+                    value = None
+                    fn0 = "get_{0}_at_v_eff_stress".format(item)  # first check for stress dependence
+                    fn1 = "get_{0}".format(item)  # first check for stress dependence
+                    if hasattr(sl, fn0):
+                        try:
+                            v_eff = self.get_v_eff_stress_at_depth(cum_thickness)
+                        except TypeError:
+                            raise ValueError("Cannot compute vertical effective stress at depth: {0}".format(cum_thickness))
+                        value = sf.get_value_of_a_get_method(sl, fn0, extras={"saturated": saturated,
+                                                                              'v_eff_stress': v_eff})
+                    elif hasattr(sl, fn1):
+                        value = sf.get_value_of_a_get_method(sl, fn1, extras={"saturated": saturated})
+                    elif hasattr(sl, item):
+                        value = getattr(sl, item)
+
+                    dd[item].append(value)
+
+        for item in dd:
+            dd[item] = np.array(dd[item])
+        self.split = dd
+
+
+def discretize_soil_profile(sp, incs=None, target=1.0):
+    """
+    Splits the soil profile into slices and stores as dictionary
+
+    :param sp: SoilProfile
+    :param incs: array_like, increments of depth to use for each layer
+    :param target: target depth increment size
+    :return: dict
+    """
+
+    if incs is None:
+        incs = np.ones(sp.n_layers) * target
+    dd = {}
+    dd["thickness"] = []
+    dd["unit_mass"] = []
+    dd["shear_vel"] = []
+    cum_thickness = 0
+    for i in range(sp.n_layers):
+        sl = sp.layer(i + 1)
+        thickness = sp.layer_height(i + 1)
+        n_slices = max(int(thickness / incs[i]), 1)
+        slice_thickness = float(thickness) / n_slices
+        for j in range(n_slices):
+            cum_thickness += slice_thickness
+            if cum_thickness >= sp.gwl:
+                rho = sl.unit_sat_mass
+                saturation = True
+            else:
+                rho = sl.unit_dry_mass
+                saturation = False
+            if hasattr(sl, "get_shear_vel_at_v_eff_stress"):
+                v_eff = sp.vertical_effective_stress(cum_thickness)
+                vs = sl.get_shear_vel_at_v_eff_stress(v_eff, saturation)
+            else:
+                vs = sl.calc_shear_vel(saturation)
+            dd["shear_vel"].append(vs)
+            dd["unit_mass"].append(rho)
+            dd["thickness"].append(slice_thickness)
+    for item in dd:
+        dd[item] = np.array(dd[item])
+    return dd
+
+
 # TODO: extend to have LiquefiableSoil
+
+
+class SoilCritical(CriticalSoil):
+    def __init__(self, pw=9800):
+        """Deprecated. Use CriticalSoil"""
+        deprecation("SoilCritical class is deprecated (remove in version 1.0), use CriticalSoil.")
+        super(SoilCritical, self).__init__(pw=pw)
+
+
+class SoilStressDependent(StressDependentSoil):
+    def __init__(self, pw=9800):
+        """Deprecated. Use StressDependentSoil"""
+        deprecation("SoilStressDependent class is deprecated (remove in version 1.0), use StressDependentSoil.")
+        super(SoilStressDependent, self).__init__(pw=pw)
