@@ -1,7 +1,8 @@
 from collections import OrderedDict
-from sfsimodels.models import SoilProfile, Foundation, Building, BuildingSDOF
+from sfsimodels.models import SoilProfile, Foundation, Building, SDOFBuilding
 from sfsimodels.exceptions import ModelError
 from sfsimodels import functions as sf
+import uuid
 
 
 # class SoilSystem(object):
@@ -53,8 +54,9 @@ class SoilStructureSystem(object):
     name = None
     base_type = "system"
     type = "sfs"
+    _unique_hash = None
     _sp = SoilProfile()
-    _bd = BuildingSDOF()
+    _bd = SDOFBuilding()
     _fd = Foundation()
     _soil_profile_id = None
     _building_id = None
@@ -63,7 +65,7 @@ class SoilStructureSystem(object):
     def __init__(self):
         self.inputs = ["id", "name", "soil_profile_id", "building_id", "foundation_id"]
 
-    def to_dict(self):
+    def to_dict(self, **kwargs):
         outputs = OrderedDict()
         skip_list = []
         for item in self.inputs:
@@ -139,3 +141,9 @@ class SoilStructureSystem(object):
         # Could add assertions here for type?
         self._bd = obj
         self._building_id = obj.id
+
+    @property
+    def unique_hash(self):
+        if self._unique_hash is None:
+            self._unique_hash = uuid.uuid1()
+        return self._unique_hash
